"""
Alias module for phylozoo.core.network.dnetwork.

This module provides an alias to allow imports like:
    from phylozoo.core.dnetwork import DirectedPhyNetwork

instead of:
    from phylozoo.core.network.dnetwork import DirectedPhyNetwork
"""

from .network.dnetwork import *  # noqa: F403, F401
