"""
Alias module for phylozoo.core.network.sdnetwork.

This module provides an alias to allow imports like:
    from phylozoo.core.sdnetwork import SemiDirectedPhyNetwork

instead of:
    from phylozoo.core.network.sdnetwork import SemiDirectedPhyNetwork
"""

from .network.sdnetwork import *  # noqa: F403, F401
